/*
 * MP3 Player, ATAPI support.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Contact: paul@pjrc.com
 *
 * File		: atapip.h (Private header file)
 * Author	: MBA
 * Created	: dec, 2001
 *
 */

#include "malloc.h"
#include "Ata.h"

//#define ATAPI_DEBUG 1

#ifndef _ATAATAPIP_H_ 
#define _ATAATAPIP_H_ 

typedef enum _XFER_MODE {_iXFER_NONE, _iXFER_READ, _iXFER_WRITE};

//typedef unsigned char STRING_Z;

// Define _BP (Byte Positions) according to the ATAPI specs & define the offsets into the data for these _BP's
// Note that _BP1 is the first, not _BP0 !!
// Normally I would use const's, but SDCC does NOT like them (VERY inefficient...)
#define _BP1     0
#define _BP2     1
#define _BP3     2
#define _BP4     3
#define _BP5     4
#define _BP6     5
#define _BP7     6
#define _BP8     7
#define _BP9     8
#define _BP10    9
#define _BP11   10
#define _BP12   11
#define _BP13   12
#define _BP14   13
  
#define _BP26   25
#define _BP33   32
#define _BP34   33
  
#define _BP89   88
#define _BP90   89
#define _BP91   90

#define _BP133 132
#define _BP134 133
#define _BP135 134
#define _BP136 135

#define _BP141 140
#define _BP142 141
#define _BP143 142
#define _BP144 143
  
#define _BP159 158
#define _BP160 159
#define _BP161 160
#define _BP162 161
  
// Bit definitions of _BP26 (File Flags) in the directory record
#define _BP26_Existence      0x01
#define _BP26_Directory      0x02
#define _BP26_Record         0x04
#define _BP26_Protection     0x08
#define _BP26_AssociatedFile 0x10
#define _BP26_Reserved       0x60
#define _BP26_MultiExtent    0x80

typedef struct
{
  STRING_Z      szName[64];
  unsigned long ulExtent;
  unsigned long ulUserData;
  
  // Public part stops here. Private part continues...
  int           iPDN;
  unsigned long ulDataLength;
  simm_id       sidNext;
} /*_AtapiDirStructType, *_PATAPI_DIR_STRUCT, */_AtapiDirStructType, *_AtapiDirStructTypePtr;

typedef struct
{
  STRING_Z      szName[64];
  unsigned long ulExtent;
  unsigned long ulDataLength;
  
  // Public part stops here. Private part continues...
  simm_id       sidNext;
} /*_AtapiFileStructType, *_PATAPI_FILE_STRUCT,*/ _AtapiFileStructType, *_AtapiFileStructTypePtr;


typedef struct
{
  unsigned char	ucYear;
  unsigned char	ucMonth;
  unsigned char	ucDay;
  unsigned char	ucHour;
  unsigned char	ucMinute;
  unsigned char	ucSecond;
  signed   char	cGreenwich;
} _AtapiFileTimeStructType, *_AtapiFileTimeStructTypePtr;

typedef struct
{
  unsigned char aucYear[4];
  unsigned char aucMonth[2];
  unsigned char aucDay[2];
  unsigned char aucHour[2];
  unsigned char aucMinute[2];
  unsigned char aucSecond[2];
  unsigned char aucHundredth[2];
  signed   char	cGreenwich;
} _AtapiVdTimeStructType, *_AtapiVdTimeStructTypePtr;

typedef struct
{
  unsigned char ucRecordLen;
  unsigned char	ucExtAttributeLen;
  unsigned long ulLocationExtent;
  unsigned long	ulDataLen;
  _AtapiFileTimeStructType tDateTime;
  unsigned char	ucFileFlags;
  unsigned char	ucFileUnitSize;
  unsigned char	ucInterleaveGapSize;
  unsigned int	uiVolumeSequenceNumber;
  unsigned char	ucFileIdentifierLen;
  STRING_Z      szFileIdentifier[256];
} _AtapiDirRecordStructType, *_AtapiDirRecordStructTypPtr;


// ***************************************************
// ********** VOLUME DESCRIPTOR DEFINITIONS **********
// ***************************************************

// Define indexes in Volume Descriptor Set, filled during the VRS (Volme Recognition Sequence)
// Assumption is that there is only 1 PVD and 1 SVD...
#define _VDSPVD 0			// Primary Volume Descriptor
#define _VDSSVD 1			// Secondary Volume Descriptor

#define _ucVOLUME_DESCR_TYPE_BOOT      0x00	// Boot record
#define _ucVOLUME_DESCR_TYPE_PVD       0x01	// Primary Volume Descriptor (at least 1)
#define _ucVOLUME_DESCR_TYPE_SVD       0x02	// Secondary Volume Descriptor (zero or more)
#define _ucVOLUME_DESCR_TYPE_VPD       0x03	// Volume Partition Descriptor
#define _ucVOLUME_DESCR_TYPE_DELIMITER 0xFF	// End of sequence

// Define FileSystemFormats for the SVD.
// Only DOS & ISO Joliet are supported at this time, Romeo and Rock Ridge are NOT supported:
#define _ucFSF_DOS         0x00		// Plain old DOS
#define _ucFSF_JOLIET_LVL1 0x40		// ISO Joliet UCS-2 Level 1
#define _ucFSF_JOLIET_LVL2 0x43		// ISO Joliet UCS-2 Level 2
#define _ucFSF_JOLIET_LVL3 0x45		// ISO Joliet UCS-2 Level 3

typedef struct
{
  unsigned char             ucVolumeDescriptorType;
  unsigned char             ucStructureVersion;
  unsigned long             ulVolumeSpaceSize;
  unsigned int              uiVolumeSetSize;
  unsigned int              uiVolumeSequenceNumber;
  unsigned int              uiLogicalBlockSize;
  unsigned long             ulPathTableSize;
  unsigned long		    ulLocationLPathTable;
  unsigned long             ulLocationOptionLPathTable;
  unsigned long             ulLocationMPathTable;
  unsigned long             ulLocationOptionMPathTable;
  _AtapiDirRecordStructType dirRootDir;
/*
  STRING_Z	            szBootSystemIdentifier[32];
  STRING_Z	            szBootIdentifier[32];
  STRING_Z	            szSystemIdentifier[32];
  STRING_Z	            szVolumeIdentifier[32];
  STRING_Z                  szVolumeSetIdentifier[128];
  STRING_Z	            szPublisherIdentifier[128];
  STRING_Z	            szPreparerIdentifier[128];
  STRING_Z	            szApplicationIdentifier[128];
  STRING_Z	            szCopyrightIdentifier[37];
  STRING_Z	            szAbstractIdentifier[37];
  STRING_Z	            szBibliographicIdentifier[37];
  _AtapiVdTimeStructType    dtCreationDateTime;
  _AtapiVdTimeStructType    dtModificationDateTime;
  _AtapiVdTimeStructType    dtExpirationDateTime;
  _AtapiVdTimeStructType    dtEffectiveDateTime;
 */
   unsigned char	    ucFileSystemFormat;
} _AtapiVolumeDescriptorStructType, *_AtapiVolumeDescriptorStructTypePtr;

typedef _AtapiVolumeDescriptorStructType _AtapiVolumeDescriptorSet[ATA_DEVICE_COUNT_MAX];




/**************************************************************/
/**************************************************************/

void check_ide();
int ata_init(void);

int read_data();

void move(int step);

#define buffersize 512

// Extended error information returned by
// reg_reset(), reg_non_data(), reg_pio_data_in() and reg_pio_data_out().

struct REG_CMD_INFO
{
   unsigned char flg;         // see TRC_FLAG_xxx below
   unsigned char ct;          // see TRC_TYPE_xxx below
   unsigned char cmd;         // command code
   unsigned char fr1;         // feature reg before
   unsigned char sc1;         // sec cnt before
   unsigned char sn1;         // sec num before
   unsigned char cl1;         // cyl low before
   unsigned char ch1;         // cyl high before
   unsigned char dh1;         // device head before
   unsigned char dc1;         // device control before
   unsigned char ec;          // detailed error code
   unsigned char to;          // not zero if time out error
   unsigned char st2;         // status reg after
   unsigned char as2;         // alt status reg after
   unsigned char er2;         // error reg after
   unsigned char sc2;         // sec cnt after
   unsigned char sn2;         // sec num after
   unsigned char cl2;         // cyl low after
   unsigned char ch2;         // cyl high after
   unsigned char dh2;         // device head after
   int totalBytesXfer;       // total bytes transfered
   unsigned int failbits;     // failure bits (ATAPI protocol errors
      #define FAILBIT15 0x8000   // extra interrupts detected
      #define FAILBIT14 0x4000
      #define FAILBIT13 0x2000
      #define FAILBIT12 0x1000
      #define FAILBIT11 0x0800
      #define FAILBIT10 0x0400
      #define FAILBIT9  0x0200
      #define FAILBIT8  0x0100   // SC( CD/IO bits) wrong at end of cmd
      #define FAILBIT7  0x0080   // byte count odd at data packet xfer time
      #define FAILBIT6  0x0040   // byte count wrong at data packet xfer time
      #define FAILBIT5  0x0020   // SC (IO bit) wrong at data packet xfer time
      #define FAILBIT4  0x0010   // SC (CD bit) wrong at data packet xfer time
      #define FAILBIT3  0x0008   // byte count wrong at cmd packet xfer time
      #define FAILBIT2  0x0004   // SC wrong at cmd packet xfer time
      #define FAILBIT1  0x0002   // got interrupt before cmd packet xfer
      #define FAILBIT0  0x0001   // slow setting BSY=1 or DRQ=1 after AO cmd
} ;

extern struct REG_CMD_INFO reg_cmd_info;

#pragma CALLEE-SAVES _CLong
unsigned long _CLong (xdata unsigned char *p_ucBytes);
#pragma CALLEE-SAVES _CInt
unsigned int _CInt (unsigned char *p_ucBytes);
#pragma CALLEE-SAVES _Long2Char
void _Long2Char (unsigned long p_ulValue, unsigned char *p_acBytes);
void _AtapiDebugPrintSectorBuffer(void);
void _AtapiGetVdsData(_AtapiVolumeDescriptorStructTypePtr p_pCurVds, unsigned char xdata *p_aucBuffer);
#pragma CALLEE-SAVES _AtapiValidateName
void _AtapiValidateName(STRING_Z* p_szName, unsigned char p_ucLen);
#endif
